// This file is part of MultiVnc-Qt.  MultiVnc-Qt is copyright 2012-2022
// by Robert W. Babcock and WSS Division of DDC and is distributed under
// the terms of the GNU General Public License.  Contact the author at
// wssddc@wssddc.com or see the project web page at
// <https://www.wssddc.com/multivnc/>.
//
// MultiVnc-Qt uses the open source Qt toolkit from the Qt Project
// (qt-project.org) and The Qt Company (www.qt.io).
//
// MultiVnc-Qt is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// MultiVnc-Qt is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with MultiVnc-Qt.  If not, see <https://www.gnu.org/licenses/>.

#include "linecompleter.h"
#include "mainwindow.h"
#include "pingresult.h"
#include "ui_pingresult.h"

#include <QCompleter>
#include <QLineEdit>
#include <QKeyEvent>
#include <QtDebug>
#include <QSettings>
#include <QAbstractItemView>
#include <QStringListModel>

// windef.h in MSVC (but not MinGW) has macros which conflict with std::min/max
// Undefine these so std::min/max below don't generate errors
#undef min
#undef max

// This behaves like a stock QLineEdit except it performs special actions
// on certain keystrokes if a QStringListModel-based QCompleter is assigned
// (and will fail horribly if some other QCompleter is assigned):
// 1. If DEL is pressed while a completion is highlighted, that completion
//    is deleted from the QSettings used to generate the completion list.
//    There's also a QSettings (registry) deletion specific to MultiVNC-Qt.
// 2. If RETURN or ENTER is hit, the completion is applied and a TAB keypress
//    is simulated to move off of the input fileld.
// 3. If INSERT is hit when no completion is highlighted (including when no text
//    has been entered), the entire list of possible completions is shown, not
//    just the ones matching entered text.

#define CLEAR_BUTTON_PIXELS	32

LineCompleter::LineCompleter(QWidget *parent) : QLineEdit(parent)
	{
	connect(this, SIGNAL(editingFinished()), this, SLOT(slot_resizeToContent()));
	}

void LineCompleter::slot_resizeToContent()
	{
	QFontMetrics metric(this->font());
// QFontMetrics.width() is deprecated
//	int pixelsWide = metric.width(this->text()) + CLEAR_BUTTON_PIXELS;
	int pixelsWide = metric.boundingRect(this->text()).width() + CLEAR_BUTTON_PIXELS;
	this->setMinimumWidth(pixelsWide);
	this->adjustSize();
//	mwptr->resize(mwptr->minWidth, 0);
	mwptr->beatOnSizeUntilItFits();
	}

void LineCompleter::keyPressEvent(QKeyEvent *e)
	{
	QCompleter *c = this->completer();
	if(!c)
		{
		QLineEdit::keyPressEvent(e);
		return;
		}

// Set the popup to be as wide as the plotWidget so it's less likely
// that host names will be truncated.
	c->popup()->setMinimumWidth(mwptr->pingResults[0]->ui->plotWidget->width());

	switch(e->key())
		{
		case Qt::Key_Delete:
			{
			int currentRow = c->popup()->currentIndex().row();
// If nothing is highlighted in the popup, currentRow will be -1
// and we don't have to do anything.
			if(currentRow < 0)
				break;
			c->setCurrentRow(currentRow);
			QString currentComp = c->currentCompletion();
// Delete registry keys. User must eventually clear or edit the input (else host gets
// added again with special settings deleted).
			QSettings mvc("WSS-DDC", "MultiVNC-Qt\\perMachine");
			mvc.remove(currentComp);

// Remove from the completer model
			QStringListModel *model = static_cast<QStringListModel *>(c->model());
// We need the row to remove in the model, but c->popup()->currentIndex().row() is
// the row in the popup.  It's not obvious how to get the model row from the
// popup row, so we just search the model for the value highlighted in the popup.
// The simplest way to do this is with indexOf, but that does a linear search
// which doesn't take advantage of the fact that the model is sorted.  So instead
// we use std::equal_range which does a binary search.  No profiling has been
// done to see if the binary search is really faster.

// Went back to using indexOf because using equal_range gave non-repeatable crashes
//			QStringList::iterator itb = model->stringList().begin();
//			QStringList::iterator ite = model->stringList().end() - 1;
//			std::pair<QStringList::Iterator, QStringList::Iterator> range =
//				std::equal_range(itb, ite, currentComp);
//			int rowToRemove = range.first - itb;

			int rowToRemove = model->stringList().indexOf(currentComp);
//			qDebug() << "old/new row number: " << oldway << rowToRemove;
			if(rowToRemove >= 0)
				model->removeRow(rowToRemove);

// Highlight the item below the one we just deleted, or the new last item if
// we just deleted the last item.  If nothing matches the text typed in,
// the completer popup goes away.
			int newRow = std::min(currentRow, c->completionCount() - 1);
			if(newRow >= 0)
				c->popup()->setCurrentIndex(c->completionModel()->index(newRow, 0));
			c->complete();
			break;
			}

// Make enter look like tab so focus moves off field after applying the completion
		case Qt::Key_Return:
		case Qt::Key_Enter:
			{
			QKeyEvent newKeyEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier);
			QCoreApplication::sendEvent(this->parent(), &newKeyEvent);
			break;
			}

// Insert pops up all possible completions if no completion is highlighted.
// Any text typed into the field is lost.
		case Qt::Key_Insert:
			{
			if(c->popup()->currentIndex().row() < 0)
				{
				c->setCompletionPrefix(QString());
				c->complete();
				}
			}
			break;

		default:
			break;
		}

	QLineEdit::keyPressEvent(e);
	return;
	}
