// This file is part of MultiVnc-Qt.  MultiVnc-Qt is copyright 2012-2022
// by Robert W. Babcock and WSS Division of DDC and is distributed under
// the terms of the GNU General Public License.  Contact the author at
// wssddc@wssddc.com or see the project web page at
// <https://www.wssddc.com/multivnc/>.
//
// MultiVnc-Qt uses the open source Qt toolkit from the Qt Project
// (qt-project.org) and The Qt Company (www.qt.io).
//
// MultiVnc-Qt is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// MultiVnc-Qt is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with MultiVnc-Qt.  If not, see <https://www.gnu.org/licenses/>.

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "configuration.h"

#include <QMainWindow>
#include <QList>
#include <QPushButton>
#include <QFormLayout>
#include <QUrl>
#include <QSoundEffect>
#include <QCompleter>
#include <QStringListModel>
#include <QHostAddress>
#include <QNetworkReply>

#define _WIN32_DCOM
#include <WinSock2.h>
#include <Windows.h>
#include <comdef.h>
#include <WbemIdl.h>
//#include <wincred.h>
//#include <strsafe.h>

enum buttonsColors { DEFAULT_COLOR, UP_COLOR, WOL_COLOR, DOWN_COLOR };
enum deleteGroup { TARGET_ALL, TARGET_DOWN, TARGET_UP };
enum VNCpasswordType { VNC_COMMON, VNC_MS_DOMAIN, VNC_CUSTOM, VNC_DEFAULT };
enum targetState { STATE_UP, STATE_DOWN, STATE_INVALID };
enum viewerType { VIEW_LOGGEDIN, VIEW_UPTIME};
enum warnSetting { WARN_NO=0, WARN_YES=1, WARN_DEFAULT=2 };

#define KEYSTROKE_DELAY_MS	10
#define CAD_DELAY_MS			2500

// VkKeyScanEx returns shift state in high byte
#define VK_SHIFT_PRESSED	0x100
#define VK_CONTROL_PRESSED	0x200
#define VK_ALT_PRESSED		0x400
#define VK_HANKAKU_PRESSED	0x800
// high byte = 16 and 32 are reserved

#define nOS 8
#define OS_DEFAULT	"Default"
#define OSNAMES "Windows 7," \
					 "Windows 8," \
					 "Windows 10," \
					 "Windows 11," \
					 "2008 Server," \
					 "2012 Server," \
					 "2016 Server," \
					 "2019 Server"

QString encryptString(QString plaintext);
QString decryptString(QString cyphertext);
QString IPtoQString(unsigned long ipaddr);
#if defined(__MINGW32__)
	BSTR __stdcall ConvertStringToBSTR(const char *const ascii);
#endif //defined(__MINGW32__)
char * __stdcall ConvertBSTRToString(BSTR bstr);

namespace Ui
	{
	class MainWindow;
	}

class PingResult;
class PingTask;

#ifndef EXTERN
	#define EXTERN extern
#endif
EXTERN bool globalEnabled;
EXTERN bool ipInVNCtitle;
EXTERN HWND foundWindow;
EXTERN MainWindow *mwptr;
EXTERN QString wantedTitle;

class MainWindow : public QMainWindow
	{
	Q_OBJECT

public:
	explicit MainWindow(QWidget *parent = nullptr);
	~MainWindow() override;
	Ui::MainWindow *ui;
	QString getVNCpath();
	void setVNCpath(QString path);
	QString getHWlistPath();
	void setHWlistPath(QString path);
	QString getMSLogonName();
	void setMSLogonName(QString name);
	QString getVNCScale();
	void setVNCScale(QString scale);
	QString getMSLogonPassword();
	void setMSLogonPassword(QString pw);
	QString getVNCpassword();
	void setVNCpassword(QString pw);
	QString getWelcomePassword();
	void setWelcomePassword(QString pw);
	int getPingCycleTime();
	void setPingCycleTime(int time);
	int getPingWaitTime();
	void setPingWaitTime(int time);
	int getMaxRows();
	void setMaxRows(int maxr);
	int getMaxCols();
	void setMaxCols(int maxc);
	int getNgraph();
	void setNgraph(int ngr);
	int getMaxSession();
	void setMaxSession(int nses);
	bool getViewOnlySmall();
	bool getDefaultMSLogon();
	void setDefaultMSLogon(bool logonsetting);
	void setViewOnlySmall(bool view);
	bool getAllowDuplicates();
	void setAllowDuplicates(bool dupe);
	bool getHidePasswords();
	void setHidePasswords(bool hide);
	int getDefaultOS();
	void setDefaultOS(int defaultval);
	QList<PingResult *> pingResults;
	QList<PingTask *> pingTasks;
	void forceOnscreen(QWidget * widget);
	int isAnyoneLoggedIn(QString host, QString uname, QString pw, QString *result);
	int upTime(QString host, QString uname, QString pw, QString *result);
	int wmicQuery(bstr_t alias, LPCWSTR property, QString host, QString uname, QString pw, QString *result);
	int isAnyoneLoggedInByRDP(QString host, QString uname, QString pw, QString *result);
	void showUserList(viewerType vt);
	void setVolume(int newvol);
	double getVolume();
	void playBing();
	void populateHistory();
	void beatOnSizeUntilItFits();
	void loadHostSettings(QString host, int taskno);

public:
	QStringList osList;
	QStringList userList;
	int activateDelay;
	int keystrokeDelay;
	int lockingCadDelay;
	int loginCadDelay;
	int loginNameDelay;
	int logoffCadDelay;
	int minWidth;
	int unlockingCadDelay;
	int warningDismissDelay;
	int welcomePositionDelay;
	int welcomeSelectDelay;
	bool defaultMSLogon;
	warnSetting globalWarn;
	QCompleter *pastHostsCompleter = nullptr;
	QStringListModel *pastHostsModel;

private:
	void aboutMVNC();
	void clipTarget(deleteGroup wantedState);
	void closeEvent(QCloseEvent *event) override;
	void createActions();
	void createMenus();
	void dragEnterEvent(QDragEnterEvent *event) override;
	void dropEvent(QDropEvent *event) override;
	void focusOffHosts();
	void gpl3License();
	void restoreSettings();
	void resumePinging(bool pingState);
	void saveSettings();
	void showDownTime(int ntask);
	void wakeOnLan(QString MAC);
	void wmicHelp();

	bool addTarget(QString host);
	bool getIP(QString host_name, PingResult *pr, PingTask *pt);
	bool isDupe(QString host);
	bool pausePinging();
	bool sessionLimit();
	bool isUnwantedIP(QHostAddress address);

	bool CoInitializeSecurityCalled;
	bool allowDuplicates;
	bool deletionInProgress;
	bool hidePasswords;
	bool soundEnabled;
	bool viewOnlySmall;

	double volume;

	int defaultOS;
	int maxCols;
	int maxRows;
	int maxSession;
	int ngraph;
	int pingCycleTime;
	int pingWaitTime;

	QSoundEffect *bing;

	QAction *aboutAct;
	QAction *aboutQtAct;
	QAction *addAct;
	QAction *clearAllAct;
	QAction *clearDownAct;
	QAction *clearLoggedinAct;
	QAction *clearUpAct;
	QAction *clipAllAct;
	QAction *clipDownAct;
	QAction *clipUpAct;
	QAction *closeAllAct;
	QAction *configAct;
	QAction *connHelpAct;
	QAction *exitAct;
	QAction *importAct;
	QAction *lockAllAct;
	QAction *logcheckAllAct;
	QAction *loginAllAct;
	QAction *logoffAllAct;
	QAction *pasteAct;
	QAction *pingAct;
	QAction *shutdownAllAct;
	QAction *soundAct;
	QAction *unlockAllAct;
	QAction *upTimeAllAct;
	QAction *verCheckAct;
	QAction *VNCallAct;
	QAction *VNCallTinyAct;
	QAction *wakeAllAct;

	QGridLayout *glayout;

	QMenu *clearMenu;
	QMenu *clipMenu;
	QMenu *fileMenu;
	QMenu *logMenu;
	QMenu *multiMenu;
	QMenu *vncMenu;

	QPushButton *addButton;
	QPushButton *configButton;
	QPushButton *soundButton;
	QPushButton *toggleButton;

	QString HWlistPath;
	QString LastImportFile;
	QString MSLogonName;
	QString MSLogonPassword;
	QString VNCScale;
	QString VNCpassword;
	QString VNCpath;
	QString welcomePassword;

	configuration *configScreen;

public slots:
	void slot_configClicked();
	void slot_processPings(int ntask, unsigned long pingTime);
	void slot_slotCaller(QUrl url);

private slots:
	void slot_VNCall();
	void slot_VNCallTiny();
	void slot_aboutQt();
	void slot_addNewHost();
	void slot_clearAll();
	void slot_clearDown();
	void slot_clearLoggedIn();
	void slot_clearUp();
	void slot_clipAll();
	void slot_clipDown();
	void slot_clipUp();
	void slot_closeAll();
	void slot_delHost(int taskno);
	void slot_delHost(int taskno, bool needPause);
	void slot_exit();
	void slot_importTargets(QString importFile);
	void slot_importTargetsBrowse();
	void slot_lockAll();
	void slot_logcheckAll();
	void slot_loginAll();
	void slot_logoffAll();
	void slot_newHost(int taskno);
	void slot_pasteTargets();
	void slot_replyFinished(QNetworkReply *);
	void slot_sendWOL(int taskno);
	void slot_shutdownAll();
	void slot_togglePings();
	void slot_toggleSounds();
	void slot_unlockAll();
	void slot_upTimeAll();
	void slot_verCheck();
	void slot_wakeAll();
	};

#endif // MAINWINDOW_H
