// This file is part of MultiVnc-Qt.  MultiVnc-Qt is copyright 2012-2022
// by Robert W. Babcock and WSS Division of DDC and is distributed under
// the terms of the GNU General Public License.  Contact the author at
// wssddc@wssddc.com or see the project web page at
// <https://www.wssddc.com/multivnc/>.
//
// MultiVnc-Qt uses the open source Qt toolkit from the Qt Project
// (qt-project.org) and The Qt Company (www.qt.io).
//
// MultiVnc-Qt is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// MultiVnc-Qt is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with MultiVnc-Qt.  If not, see <https://www.gnu.org/licenses/>.

#include "textviewer.h"
#include "ui_textviewer.h"
#include "mainwindow.h"

#include <QFontDialog>
#include <QSettings>
#include <QDesktopWidget>
#include <QStyle>
#include <QScreen>

textViewer::textViewer(QString title, QWidget *parent) : QMainWindow(parent), ui(new Ui::textViewer)
	{
	ui->setupUi(this);
	FontTv = new QFont(ui->tvText->font());
	this->setWindowFlags(Qt::CustomizeWindowHint | Qt::Dialog | Qt::WindowCloseButtonHint | Qt::WindowTitleHint);
	ui->tvText->setOpenExternalLinks(true);
	this->setWindowTitle(title);
	restoreSettings();
	connect(ui->pushButton_closeTextViewer, SIGNAL(released()), this, SLOT(slot_closeClicked()));
	connect(ui->pushButton_textFont, SIGNAL(released()), this, SLOT(slot_fontClicked()));
	saveSize = true;
	mwptr->forceOnscreen(this);
	}

textViewer::~textViewer()
	{
	delete ui;
	delete FontTv;
	}

// Scroll the view to the top of the displayed text
void textViewer::scrollTop()
	{
	ui->tvText->moveCursor(QTextCursor::Start);
	ui->tvText->ensureCursorVisible();
	}

void textViewer::slot_closeClicked()
	{
	slot_saveSettings();
	this->close();
	}

// If caller is setting the size, don't save the size
void textViewer::setSizeSaving(bool saveit)
	{
	saveSize = saveit;
	}

void textViewer::slot_saveSettings()
	{
	QSettings settings("WSS-DDC", "MultiVNC-Qt\\Settings");
	settings.setValue("TvGeometryPos", pos());
	if(saveSize)
		settings.setValue("TvGeometrySize", size());
	settings.setValue("FontTv", *FontTv);
	}

void textViewer::restoreSettings()
	{
	QSettings settings("WSS-DDC", "MultiVNC-Qt\\Settings");
	if(settings.contains("FontTv"))
		{
		FontTv->fromString(settings.value("FontTv").toString());
		ui->tvText->setFont(*FontTv);
		}

	if(settings.contains("TvGeometrySize"))
		{
		QSize size = settings.value("TvGeometrySize").toSize();
		resize(size);
		}
	if(settings.contains("TvGeometryPos"))
		{
		QPoint pos = settings.value("TvGeometryPos").toPoint();
		move(pos);
		}
// The default position centers textViewer on the main window.
// If the main windows is near the top of the screen, as is natural
// for this application, the viewer extends off the top of the screen.
// So if position hasn't been saved, center the widget.
	else
		{
		QDesktopWidget *homeWidget = reinterpret_cast<QDesktopWidget *>(mwptr->centralWidget());
		this->setGeometry(QStyle::alignedRect(Qt::LeftToRight, Qt::AlignCenter,
				  this->size(), QGuiApplication::screens()[homeWidget->screenNumber(homeWidget)]->geometry()
			 )
		);
		}
	}

void textViewer::slot_fontClicked()
	{
	bool ok;
	QFont font = QFontDialog::getFont(&ok, *FontTv);
	if(ok)
		{
		*FontTv = font;
		ui->tvText->setFont(*FontTv);
// Save the font change immediately so it is in effect if another textviewer
// is opened from within this one.
		QSettings settings("WSS-DDC", "MultiVNC-Qt\\Settings");
		settings.setValue("FontTv", *FontTv);
		}
	}
